#!/bin/bash
VER=1.1
###########################################################
# Tur-FolderLimit by Turranius    (turranius@hotmail.com) #
# Idea by soehest. Go fbsd boy!                           #
#---------------------------------------------------------#
# There are two possible reasons for this script:         #
# 1: You want a nice clean incoming folder with not too   #
#    many folders in them.                                #
# 2: You run two sections on the same partition and want  #
#    to make sure one of the sections does not take up    #
#    too much space.                                      #
#                                                         #
# This script check how many folders there are in the     #
# sections you define. Once there are more then the limit #
# you set, it will move the oldest ones to another        #
# location that you select.                               #
# Works excellent for moving dated dirs too of course.    #
#                                                         #
# Hint: This one checks the time on the folder to consider#
# which folders are the oldest. I suggest you run my      #
# 0DaysDateFix script before this one, so you are sure it #
# moves them in order (For dated dirs only of course).    #
#                                                         #
#---------------------------------------------------------#
# Installation:                                           #
# Copy this script to where ever you want (/glftpd/bin?)  #
# and make it executable.                                 #
# Set up the options below, and be sure you understand    #
# the SECTIONS part.                                      #
#                                                         #
#---------------------------------------------------------#
# Run the script with the variable "test"                 #
# ( tur-folderlimit.sh test ) to see what it will do, and #
# that everything looks ok. It will NOT move anything in  #
# test mode, so dont worry.                               #
#---------------------------------------------------------#
# Finally, crontab the script to run as often as you like.#
#                                                         #
# Set up the following settings.                          #
# LOG= Where to log when moving stuff? Set to "" to       #
# disable.                                                #
#                                                         #
# SECTIONS= Sections to move from and to, how many folders#
# are allowed in the "from" part.                         #
# Its divided into 4 different variables.                 #
# Source folder                                           #
# Destination folder                                      #
# How many folders are allowed in the Source folder top?  #
# Last variable is DEEP. With this option there, it will  #
# not move, say the folder 0406 to the archive. It will   #
# move its contents to the archive folder, and delete the #
# now empty 0406 folder.                                  #
# If you dont want this, just leave the 4th variable      #
# empty.                                                  #
#                                                         #
# You seperate the variables with a : and put one section #
# per line.  Look at the example.                         #
#                                                         #
# /glftpd/site/MP3:/glftpd/site/Archive/MP3:5:DEEP        #
# would mean it will move the CONTENTS of the olderst     #
# dirs in /glftpd/site/MP3 to /glftpd/site/Archive/MP3    #
# until there are only 5 folders left in /glftpd/site/MP3 #
# Leaving DEEP out means it simply moves the oldest       #
# folders as they are in the MP3 folder.                  #
#                                                         #
# EXCLUDE= Folders in any of the Source folders to exclude#
# This might include your pre dirs and any symlinks that  #
# points to any dated folder or something.                #                     
#                                                         #
# PRESERVE= Setting this to true will make it first copy  #
# the source to destination, then delete the source. It   #
# is a bit slower, but it will keep the modified time on  #
# the folder intact.                                      #
#---------------------------------------------------------#
# Changelog:                                              #
# 1.2 : Added the option to have the contents of the      #
#       source folder moved to a destination, instead of  #
#       the entire folder.                                #
# 1.1 : Add: Options to preserve modified date on a folder#
# 1.0 :      Initial release.                             #
#---------------------------------------------------------#
# Options:

LOG="/glftpd/ftp-data/logs/tur-folderlimit.log"

SECTIONS="
/glftpd/site/0DAYS:/glftpd/site/Archive/0DAYS:5:DEEP
/glftpd/site/MP3:/glftpd/site/Archive/MP3:5
"

EXCLUDE="Today Yesterday GROUPS"

PRESERVE="TRUE"


########################################################################
# No changes needed below here.                                        #
########################################################################

## Check date for the logfile.
DATENOW="$(date +%D" - "%T)"

## Make an | between excludes.
EXCLUDE="$( echo $EXCLUDE | tr -s ' ' '|' )"

## Do it.
for section in $SECTIONS; do

  sectionname="$( echo $section | awk -F":" '{print $1}' )"
  sectionto="$( echo $section | awk -F":" '{print $2}' )"
  LIMIT="$( echo $section | awk -F":" '{print $3}' )"
  if [ "$LIMIT" = "" ]; then
    echo "No limit found for $sectionname. Im quitting."
    exit 1
  fi
  GODEEP="$( echo $section | awk -F":" '{print $4}' )"
  if [ "$GODEEP" = "DEEP" ]; then
    GODEEP="TRUE"
  fi

  limitnow="0"

  if [ -e $sectionname ]; then
    if [ "$1" = "test" ]; then
      echo "Moving into $sectionname"
      echo "There will only be $LIMIT folders left."
    fi
  else
    echo "$sectionname can not be found. Check your SECTIONS setup."
    echo "This one is wrong: $section"
    exit 1
  fi

  if [ -e $sectionto ]; then
    if [ "$1" = "test" ]; then
      echo "Stuff will be moved into $sectionto"
    fi
  else
    echo "$sectionto can not be found. Check your SECTIONS setup."
    echo "This one is wrong: $section"
    exit 1
  fi

  cd $sectionname
  DIR="$( ls --sort=time | egrep -vw $EXCLUDE )"
  count="0"
  for dirs in $DIR; do
    count="$( expr $count \+ 1 )"
    if [ "$count" -gt "$LIMIT" ]; then
      if [ "$1" = "test" ]; then
        ## Test mode one. Just printing it to shell.
        if [ "$GODEEP" = "TRUE" ]; then
          echo "Moving contents of $dirs/* to $sectionto  (test)"
          echo "Deleting $dirs"
        else
          echo "Moving $dirs to $sectionto  (test)"
          echo "Deleting $dirs"
        fi
      else
        if [ "$PRESERVE" = "TRUE" ]; then
          ## Copy folder, delete old one.
          if [ "$GODEEP" = "TRUE" ]; then
            cp -rf $dirs/* $sectionto
          else
            cp -rf $dirs $sectionto
          fi
          if [ "$GODEEP" != "TRUE" ]; then
            if [ -e $sectionto/$dirs ]; then
              rm -rf $dirs
            else
              echo "Hmm, copying of $dirs to $sectionto seems to have crapped out"
              echo "Im quitting!"
              exit 1
            fi
          else
            rm -rf $dirs
          fi
          if [ "$LOG" != "" ]; then
            echo "$DATENOW : Copying, then removing $dirs into $sectionto" >> $LOG
          fi
        else
          ## Actually move folders, and log if not disabled.
          if [ "$GODEEP" = "TRUE" ]; then
            mv -f $dirs/* $sectionto
          else
            mv -f $dirs $sectionto
          fi
          if [ "$LOG" != "" ]; then
            if [ "$GODEEP" = "TRUE" ]; then
              echo "$DATENOW : Moving contents of $dirs into $sectionto" >> $LOG
            else
              echo "$DATENOW : Moving $dirs into $sectionto" >> $LOG
            fi
          fi
        fi
      fi
    fi
  done

  if [ "$1" = "test" ]; then
    echo ""
  fi
  GODEEP=""
done

exit 0